/*
 * getpppinfo   This command returns the requested characteristic of
 *              a point-to-point connection. The requested characteristic can be
 *              the local or remote IP address of the link, or the receive
 *              or transmit packet count of the link.
 *              Based on ifconfig by Fred N. van Kempen.
 *
 * Author:      Robert Adkins, cWare, Inc. <radkins@cwareco.com>
 *              Copyright 1999 cWare Inc
 *
 *              This program is free software; you can redistribute it
 *              and/or  modify it under  the terms of  the GNU General
 *              Public  License as  published  by  the  Free  Software
 *              Foundation;  either  version 2 of the License, or  (at
 *              your option) any later version.
 */

#define DFLT_AF "inet"
#define _PATH_PROCNET_DEV "/proc/net/dev"

#include <features.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>

int opt_l = 0;			/* show local ppp address */
int opt_r = 0;			/* show remote ppp address */
int opt_rx = 0;			/* show remote receive packet count */
int opt_tx = 0;			/* show remote transmit packet count */

long int rx_bytes = 0;
long int rx_packets = 0;
long int tx_bytes = 0;
long int tx_packets = 0;

static char *get_name(char *name, char *p)
{
    while (isspace(*p))
	p++;
    while (*p) {
	if (isspace(*p))
	    break;
	if (*p == ':') {	/* could be an alias */
	    char *dot = p, *dotname = name;
	    *name++ = *p++;
	    while (isdigit(*p))
		*name++ = *p++;
	    if (*p != ':') {	/* it wasn't, backup */
		p = dot;
		name = dotname;
	    }
	    if (*p == '\0')
		return NULL;
	    p++;
	    break;
	}
	*name++ = *p++;
    }
    *name++ = '\0';
    return p;
}
static int get_dev_stats(char *bp, int version)
{
    long int dummy;

    switch (version) {
    case 3:
	sscanf(bp,
	"%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
	       &rx_bytes,
	       &rx_packets,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,

	       &tx_bytes,
	       &tx_packets,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy);
	break;
    case 2:
	sscanf(bp, "%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
	       &rx_bytes,
	       &rx_packets,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,

	       &tx_bytes,
	       &tx_packets,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy);
	break;
    case 1:
	sscanf(bp, "%ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld",
	       &rx_packets,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,

	       &tx_packets,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy,
	       &dummy);
	rx_bytes = 0;
	tx_bytes = 0;
	break;
    }
    return 0;
}
int if_readstats(char *ifname)
{
    FILE *fh;
    char buf[512];
    char thisName[512];
    int err;
    int version;

    fh = fopen(_PATH_PROCNET_DEV, "r");
    if (!fh) {
	perror(_PATH_PROCNET_DEV);
	return -1;
    }
    fgets(buf, sizeof buf, fh);	/* eat line */
    fgets(buf, sizeof buf, fh);

    version = 1;
    if (strstr(buf, "compressed"))
	version = 3;
    else if (strstr(buf, "bytes"))
	version = 2;

    err = 0;
    while (fgets(buf, sizeof buf, fh)) {
	char *s;

	s = get_name(thisName, buf);
	if (!strcmp(thisName, ifname)) {
	    get_dev_stats(s, version);
	    break;
	}
    }
    if (ferror(fh)) {
	perror(_PATH_PROCNET_DEV);
	err = -1;
    }
    return err;
}
void if_printout(char *ifname)
{
    struct ifreq ifr;
    int fd;

    if (opt_l || opt_r) {
	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd >= 0) {
	    strcpy(ifr.ifr_name, ifname);
	    if ((ioctl(fd, SIOCGIFFLAGS, &ifr) == 0) && ((ifr.ifr_flags & IFF_UP) == IFF_UP)) {
	      strcpy(ifr.ifr_name, ifname);
              if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
                  if (opt_l)
		    printf(inet_ntoa(*(struct in_addr *) &((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr));
                  strcpy(ifr.ifr_name, ifname);
                  if (opt_r && (ioctl(fd, SIOCGIFDSTADDR, &ifr) >= 0))
		    printf(inet_ntoa(*(struct in_addr *) &((struct sockaddr_in *) &ifr.ifr_dstaddr)->sin_addr));
              }
            }
	}
    }
    if (opt_rx || opt_tx) {
	if (if_readstats(ifname) >= 0) {
	    if (opt_rx)
		printf("%lu", rx_packets);
	    if (opt_tx)
		printf("%lu", tx_packets);
	}
    }
}
static void usage(void)
{
    fprintf(stderr, "Usage:\n  getpppinfo [-l] [-r] [-RX] [-TX] [<interface>]\n");
    exit(4);
}
int main(int argc, char **argv)
{
    char ifname[IFNAMSIZ];

    /* Find any options. */ argc--;
    argv++;
    while (argc && *argv[0] == '-') {
	if (!strcmp(*argv, "-l"))
	    opt_l = 1;
	if (!strcmp(*argv, "-r"))
	    opt_r = 1;
	if (!strcmp(*argv, "-RX"))
	    opt_rx = 1;
	if (!strcmp(*argv, "-TX"))
	    opt_tx = 1;
	if (!strcmp(*argv, "-?") || !strcmp(*argv, "-h") || !strcmp(*argv, "-help") || !strcmp(*argv, "--help"))
	    usage();
	argv++;
	argc--;
    }
    if (!opt_l && !opt_r && !opt_rx && !opt_tx)
	opt_r = 1;
    if (opt_l)
	opt_r = opt_tx = opt_rx = 0;
    else if (opt_r)
	opt_l = opt_tx = opt_rx = 0;
    if (opt_rx)
	opt_tx = 0;
    if (argc == 0)
	strncpy(ifname, "ppp0", IFNAMSIZ);
    else {
        ifname[IFNAMSIZ-1] = '\0';
	strncpy(ifname, *argv, IFNAMSIZ-1);
    }
    if_printout(ifname);
    exit(0);
}
