/*
 * xeqlinfo: X application for displaying eql load balancing
 *
 */

/* 
 * Based upon xsysinfo, written by Gabor Herr <herr@iti.informatik.th-darmstadt.de>.
 * xeqlinfo adaptation was written by cWare, Inc.
 *
 * Copyright (c) 1992, 1993 by Gabor Herr, all rights reserved.
 * Copyright (c) 1999 by cWare, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that may name is not used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission. I make no representations
 * about the suitability of this software for any purpose. It is
 * provided "as is" without express or implied warranty.
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include "BarGauge.h"
#include <stdio.h>
#include "eqlinfo.h"

#define norm(x,max)	((max) == 0 ? 0 : (double)(x) / (max))

/* Application Ressources: */
typedef struct {
	Boolean show_title;
	Boolean show_labels;
	Boolean show_eql;
	Boolean show_load;
	Boolean show_mem;
	Boolean show_swap;
} AppData, *AppDataPtr;

static AppData appres;

static XtResource application_resources[] = {
	{ "showTitle", "ShowTitle", XtRBoolean, sizeof(Boolean),
	  XtOffset(AppDataPtr,show_title), XtRImmediate, (XtPointer) True },
	{ "showLabels", "ShowLabels", XtRBoolean, sizeof(Boolean),
	  XtOffset(AppDataPtr,show_labels), XtRImmediate, (XtPointer) True },
	{ "showEql", "ShowEql", XtRBoolean, sizeof(Boolean),
	  XtOffset(AppDataPtr,show_eql), XtRImmediate, (XtPointer) True }
};

/* Command line options */
static XrmOptionDescRec options[] = {

	{ "-update", "*.bar.update", XrmoptionSepArg, NULL },

	{ "-title",   "*showTitle", XrmoptionNoArg, (XtPointer) "True" },
	{ "-notitle", "*showTitle", XrmoptionNoArg, (XtPointer) "False" },

	{ "-labels",   "*showLabels", XrmoptionNoArg, (XtPointer) "True" },
	{ "-nolabels", "*showLabels", XrmoptionNoArg, (XtPointer) "False" },

	{ "-eql",   "*showEql", XrmoptionNoArg, (XtPointer) "True" },
	{ "-noeql", "*showEql", XrmoptionNoArg, (XtPointer) "False" }

};

static String fallback_resources[] = {
	"*.title.label:		Linux Eql Info",
#if MAX_STATS > 5 
	"*.eql*label:		ppp0+:",
#elif MAX_STATS == 5 
	"*.eql*label:		ppp0-5:",
#elif MAX_STATS == 4 
	"*.eql*label:		ppp0:1:2:",
#else
	"*.eql*label:		ppp0/ppp1:",
#endif
	NULL
};

static XtAppContext context;
static struct Stats stats[XG_NUM_SEGMENTS_STATS];
static struct Stats old_stats[XG_NUM_SEGMENTS_STATS];
static struct Stats tmp_stats[XG_NUM_SEGMENTS_STATS];
static int devupdate_freq = 0;

static void update_devinfo()
{
	static int count = 0;
	int i;

	if ( count-- > 0 )
		return;

	count = devupdate_freq;
  
	get_devinfo( stats );
	for (i = 1; i < MAX_STATS; i++) {
		tmp_stats[i] = stats[i];
		stats[i].rx_packets -= old_stats[i].rx_packets;
		old_stats[i] = tmp_stats[i];
	}
}

static void geteqlCB( widget, client_data, call_data )
	Widget widget;
XtPointer client_data;
XtPointer call_data;
{
        long int tot_packets = 0;
	int i;

	XgBarGaugeValues *gauge = (XgBarGaugeValues *) call_data;

	update_devinfo();
	for (i = 1; i < MAX_STATS; i++) {
            tot_packets += stats[i].rx_packets;
        }
	for (i = 1; i < MAX_STATS; i++) {
	    gauge->values[i-1] = norm(stats[i].rx_packets,tot_packets);
            /*printf("stats[i].rx_packets=%ld, tot_packets=%ld\n",stats[i].rx_packets,tot_packets);*/
        }
}

static Widget create_gauge( name, parent, segs, valueproc, client_data )
	String name;
Widget parent;
XtCallbackProc valueproc;
int segs;
XtPointer client_data;
{
	Widget gauge;
	Widget bar;
	Widget label;
	char ppp_label[256];
  
	gauge = XtCreateManagedWidget( name, formWidgetClass, parent, NULL, 0 );
 
	if (appres.show_labels) {
#if MAX_STATS > 4 
                sprintf(ppp_label,"ppp0-%i",MAX_STATS);
#elif MAX_STATS == 4 
                sprintf(ppp_label,"ppp0:1:2",MAX_STATS);
#else
                sprintf(ppp_label,"ppp0:ppp1",MAX_STATS);
#endif
		label = XtVaCreateManagedWidget( ppp_label,labelWidgetClass,gauge,
						 XtNright, XawChainLeft,
						 NULL);

		bar = XtVaCreateManagedWidget( "bar",barGaugeWidgetClass,gauge,
					       XtNsegments, segs,
					       XtNfromHoriz, label,
					       XtNleft, XawChainLeft,
					       NULL);
	} else {
		bar = XtVaCreateManagedWidget( "bar",barGaugeWidgetClass,gauge,
					       XtNsegments, segs,
					       XtNleft, XawChainLeft,
					       NULL);
	}

	XtAddCallback( bar, XtNgetValue, valueproc, client_data );

	return bar;
}

static void create_app( top )
	Widget top;
{
	Widget form;
	Widget above_widget = NULL;
	Widget this;
	Widget bar;
  
	form = XtCreateManagedWidget( "form", formWidgetClass, top, NULL, 0 );

	if (appres.show_title)
		above_widget = XtVaCreateManagedWidget( "title",
							labelWidgetClass, form,
							XtNright, XawChainRight,
							NULL);

	if (appres.show_eql)	{
		bar  = create_gauge( "eql", form, 2, geteqlCB, NULL );
		XtVaSetValues( bar,
			       XtNautoAdjust, False,
			       NULL );
		this = XtParent(bar);
		XtVaSetValues( this,
			       XtNfromVert, above_widget,
			       XtNright,    XawChainRight,
			       NULL );
		above_widget = this;
	}
    

	if (above_widget == NULL)
		XtAppError( context,"All gauges disabled, nothing to show.\n" );
}

static void show_usage()
{
	fprintf(stderr,"xeqlinfo version 1.0\n\n");
	fprintf(stderr,"usage: xeqlinfo [options...]\n\n");
	fprintf(stderr,"where options include:\n");
#ifdef USE_MILLI_SECONDS       /* (ThMO) */
	fprintf(stderr,"-update <n>                     set update rate to <n> milli-seconds\n");
#else
	fprintf(stderr,"-update <n>                     set update rate to <n> 
seconds\n");
#endif
	fprintf(stderr,"-title   (-notitle)		show (don't show) title string\n");
	fprintf(stderr,"-labels  (-nolabels)		show (don't show) gauge labels\n");
	fprintf(stderr,"-eql 	 (-noeql)   		show (don't show) CPU load average value\n");
	fprintf(stderr,"-help				show this message\n");
	fprintf(stderr,"\nor one of the standard toolkit options.\n\n");
}

main( argc, argv )
	int argc;
char **argv;
{
	Arg args[1];
	Widget toplevel;

	toplevel = XtAppInitialize( &context, "XEqlinfo",
				    options, XtNumber(options),
				    &argc, argv,
				    fallback_resources,
				    NULL, 0 );

	if (argc != 1) {
		show_usage();
		exit(1);
	}
			      
	XtGetApplicationResources( toplevel, &appres,
				   application_resources,
				   XtNumber( application_resources ),
				   NULL, 0 );
			      
	if (eqlinfo_init() != 0) {
		fprintf(stderr,	"Proc filesystem isn't mounted on `/proc'.\n");
		exit(1);
	}
	update_devinfo();

	create_app( toplevel );

	XtRealizeWidget(toplevel);
	XtAppMainLoop(context);
}
